"""
标记价格类，用于确定标记价格的属性和方法
标记价格，更新频率比较低，mark_price参数直接使用json格式
"""


class MarkPriceData:
    """保存标记价格信息"""

    def __init__(self, mark_price_info, has_been_json_encoded=False):
        self.event = "MarkPriceEvent"
        self.mark_price_info = mark_price_info
        self.has_been_json_encoded = has_been_json_encoded

    def init_data(self):
        """
        init data from request or wss
        :return: None
        """
        raise NotImplementedError

    def get_all_data(self):
        raise NotImplementedError

    def get_event(self):
        """
        get event type
        :return: str, MarkPriceEvent
        """
        return self.event

    def get_exchange_name(self):
        """
        get exchange name
        :return: str, eg BINANCE & OKX
        """
        raise NotImplementedError

    def get_server_time(self):
        """
        get server time generated by the server
        :return: float, timestamp
        """
        raise NotImplementedError

    def get_local_update_time(self):
        """
        get local update time for the data generate
        :return: float, timestamp
        """
        raise NotImplementedError

    def get_symbol_name(self):
        """
        get the data's symbol
        :return: str, symbol's name
        """
        raise NotImplementedError

    def get_mark_price(self):
        """
        get mark_price
        :return: float, mark_price
        """
        raise NotImplementedError

    def __str__(self):
        raise NotImplementedError

    def __repr__(self):
        raise NotImplementedError
